# Demo code to connect to a Luna Lightwave Analyzer using SCPI

import socket
import matplotlib.pyplot as plt

# IP address of laptop controlling the Luna Lightwave Analyzer
# 127.0.0.1 is the local machine
ip_addr = '127.0.0.1'
# TCP/IP port for SCPI connection. Use 5025 for automated
# connections, or 5024 for a more verbose telnet connection
ip_port = 5025

# Socket connection
s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)


# Connect to Lightwave Analyzer
def connect(addr = ip_addr, port = ip_port):
     s.connect((addr, port))

     
# Send a SCPI command or query
# Return a response for queries
def cmd(s):
    # Is this a query? Does the command (before any arguments) end with '?'
    if s.split()[0].endswith('?'):
        return query(s)
    else:
        command(s)


# Send a SCPI command
# SCPI commands receive no response from the SCPI server
def command(cmd):
    cmd += '\r\n'
    s.send(cmd.encode('utf-8'))


# Is the given command a query that returns an array of data?
def is_data_query(cmd):
    ret = False
    data_queries = [':OFDR?', ':DIST?', ':SPEC?', ':WAV?']
    for q in data_queries:
        if q in cmd.upper():
            ret = True
    return ret


# Send a SCPI query and return the server response
# SCPI queries end in '?' and will receive a response from the server 
def query(cmd):
    command(cmd)
    resp = get_response()
    
    # Is this a data response that should be converted to a list?
    if is_data_query(cmd):
        resp = [float(n) for n in resp.split(',')]

    return resp


# Get a response from the SCPI server
def get_response():
    resp = ''
    # Time out if we have not received data for 5 seconds
    s.settimeout(5)
    
    # Receive data until we time out or receive a buffer that
    # contains a \n
    while True:
        r = ''
        try:
            r = s.recv(2048).decode('utf-8')
        except socket.timeout:
            break
        # Append to received data
        resp += r
        if '\n' in resp:
            break

    # Return string of received data
    return resp.rstrip()


def plot(x, y, xlabel = '', ylabel = '', title = ''):
    fig, ax = plt.subplots()
    ax.plot(x, y)
    ax.set_title(title)
    ax.set_xlabel(xlabel)
    ax.set_ylabel(ylabel)
    plt.show()

    
if __name__ == '__main__':
    # Connect to Lightwave Analyzer
    connect()
    
    # Get device identification string
    idn = cmd('*IDN?')
    print(idn)
    
    # Query group index
    grp_idx = cmd('GIND?')  
    print('Initial group index: ' + grp_idx)
    
    # Set group index
    cmd('GIND 1.5')

    # Make sure group index has changed
    grp_idx = cmd('GIND?')  
    print('New group index: ' + grp_idx)

    # Initiate a new measurement
    cmd('INIT')
    
    # Get return loss at location 1.2 meters
    rl = cmd('FETC:RL? 1.2')
    print('RL: ' + rl)
    
    # Get y-axis OFDR data for measurement
    ofdr = cmd('FETC:OFDR?')
    print('Received ' + str(len(ofdr)) + ' points of measurement data')

    # Get x-axis OFDR data for measurement
    dist = cmd('FETC:DIST?')

    # Plot OFDR data
    plot(dist, ofdr, 'Length (m)', 'Amplitude (dB)')

    # Plot OFDR data for from 1.5m to 2.5m
    cmd('CONF:OFDR 0, 1.5, 2.5')
    dist = cmd('FETC:DIST?')
    ofdr = cmd('FETC:OFDR?')
    plot(dist, ofdr, 'Length(m)', 'Amplitude (dB)')
    
    # Show current spectral cursor position and width
    print(cmd('CONF:SPEC?'))

    # Set spectral cursor to location = 1.0 m, width = 0.5m
    cmd('CONF:SPEC 1.0, 0.5')

    # Get spectral data and wavelength
    spec = cmd('FETC:SPEC?')
    wav = cmd('FETC:WAV?')
    plot(wav, spec, 'Wavelength (nm)', 'Return Loss (dB)', 'Spectral Plot')

